@@ -1,10 +1,11 @@
 // Install script for Enigmail
 
 var err;
+const APP_VERSION="0.83.3";
 
-err = initInstall("Enigmail v0.76.8",  // name for install UI
+err = initInstall("Enigmail v"+APP_VERSION,  // name for install UI
                   "/enigmail",         // registered name
-                  "0.76.8.0");         // package version
+                  APP_VERSION+".0");         // package version
 
 logComment("initInstall: " + err);
 
@@ -37,12 +38,29 @@ if (!verifyDiskSpace(fProgram, srDest)) {
 
   } else {
     // Register chrome
+    
+    var isTbird = false;
+    var execFile = 'thunderbird' + (getPlatform() == "win" ? '.exe' : '-bin');
+    if (File.exists(getFolder(getFolder('Program'), execFile))) {
+      isTbird = confirm("Installing on Thunderbird. Is this correct?");
+    }
+    else {
+      isTbird = !confirm("Installing on Mozilla or Netscape. Is this correct?");
+    }
+
+//  old way:    
+//  var isTbird = !confirm("Which Theme do you want to install for Enigmail? Click:\n[ OK ] for Mozilla\n[ Cancel ] for Thunderbird");
+    
     registerChrome(PACKAGE | DELAYED_CHROME, getFolder("Chrome","enigmail.jar"), "content/enigmail/");
 
-    registerChrome(   SKIN | DELAYED_CHROME, getFolder("Chrome","enigmail.jar"), "skin/modern/enigmail/");
-
-    registerChrome(   SKIN | DELAYED_CHROME, getFolder("Chrome","enigmail.jar"), "skin/classic/enigmail/");
+    if (! isTbird) {
+      registerChrome(   SKIN | DELAYED_CHROME, getFolder("Chrome","enigmail-skin.jar"), "skin/modern/enigmail/");
 
+      registerChrome(   SKIN | DELAYED_CHROME, getFolder("Chrome","enigmail-skin.jar"), "skin/classic/enigmail/");
+    }
+    else {
+      registerChrome(   SKIN | DELAYED_CHROME, getFolder("Chrome","enigmail-skin-tbird.jar"), "skin/classic/enigmail/");
+    }
     registerChrome( LOCALE | DELAYED_CHROME, getFolder("Chrome","enigmail.jar"), "locale/en-US/enigmail/");
 
     err = getLastError();
@@ -52,6 +70,9 @@ if (!verifyDiskSpace(fProgram, srDest)) {
 
     } else {
       performInstall();
+      if (isTbird) {
+        alert("Enigmail v"+APP_VERSION+" has been successfully installed. Restart after EnigMime has been installed as well.");
+      }
     }
   }
 }
