@@ -1,8 +1,41 @@
+/*
+The contents of this file are subject to the Mozilla Public
+License Version 1.1 (the "MPL"); you may not use this file
+except in compliance with the MPL. You may obtain a copy of
+the MPL at http://www.mozilla.org/MPL/
+
+Software distributed under the MPL is distributed on an "AS
+IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
+implied. See the MPL for the specific language governing
+rights and limitations under the MPL.
+
+The Original Code is Enigmail.
+
+The Initial Developer of the Original Code is Ramalingam Saravanan.
+Portions created by Ramalingam Saravanan <svn@xmlterm.org> are
+Copyright (C) 2001 Ramalingam Saravanan. All Rights Reserved.
+
+Contributor(s):
+Patrick Brunschwig <patrick.brunschwig@gmx.net>
+
+Alternatively, the contents of this file may be used under the
+terms of the GNU General Public License (the "GPL"), in which case
+the provisions of the GPL are applicable instead of
+those above. If you wish to allow use of your version of this
+file only under the terms of the GPL and not to allow
+others to use your version of this file under the MPL, indicate
+your decision by deleting the provisions above and replace them
+with the notice and other provisions required by the GPL.
+If you do not delete the provisions above, a recipient
+may use your version of this file under either the MPL or the
+GPL.
+*/
+
 // enigmailCommon.js: shared JS functions for Enigmail
 
 // This Enigmail version and compatible Enigmime version
-var gEnigmailVersion = "0.76.8.0";
-var gEnigmimeVersion = "0.76.8.0";
+var gEnigmailVersion = "0.83.3.0";
+var gEnigmimeVersion = "0.83.3.0";
 
 // Maximum size of message directly processed by Enigmail
 const ENIG_MSG_BUFFER_SIZE = 96000;
@@ -37,6 +70,7 @@ const ENIG_IOSERVICE_CONTRACTID = "@mozilla.org/network/io-service;1";
 
 const ENIGMAIL_PREFS_ROOT       = "extensions.enigmail.";
 
+
 // Interfaces
 const nsIEnigmail               = Components.interfaces.nsIEnigmail;
 const nsIEnigStrBundle          = Components.interfaces.nsIStringBundleService;
@@ -59,17 +93,10 @@ const ENIG_TEMPDIR_PROP = "TmpD";
 var gUsePGPMimeOptionList = ["usePGPMimeNever", "usePGPMimePossible",
                              "usePGPMimeAlways"];
 
-var gEnigDefaultEncryptionOptions = ["defaultEncryptionNone",
-                                     "defaultEncryptionOnly",
-                                     "defaultEncryptionSign"];
-
 var gEnigRecipientsSelectionOptions = ["askRecipientsNever",
                                        "askRecipientsClever",
                                        "askRecipientsAlways"];
 
-var gEnigImmediateSendOptions = ["sendLater",
-                                 "sendNow"];
-
 const ENIG_BUTTON_POS_0           = 1;
 const ENIG_BUTTON_POS_1           = 1 << 8;
 const ENIG_BUTTON_POS_2           = 1 << 16;
@@ -81,41 +108,43 @@ const ENIG_THREE_BUTTON_STRINGS   = (ENIG_BUTTON_TITLE_IS_STRING * ENIG_BUTTON_P
 
 
 var gEnigmailPrefDefaults = {"configuredVersion":"",
-                             "logDirectory":"",
-                             "initAlertCount":2,
-                             "composeHtmlAlertCount":3,
                              "agentPath":"",
-                             "autoCrypto":false,
-                             "useDefaultComment":false,
-                             "userIdValue":"",
-                             "userIdFromAddr":false,
-                             "noPassphrase":false,
-                             "usePGPMimeOption":PGP_MIME_POSSIBLE,
-                             "mimeHashAlgorithm":1,
-                             "defaultEncryptionOption":1,
-                             "defaultSignMsg":false,
-                             "defaultSignNewsMsg":false,
                              "alwaysTrustSend":true,
-                             "encryptToSelf":true,
-                             "confirmBeforeSend":false,
-                             "doubleDashSeparator":false,
-                             "maxIdleMinutes":5,
-                             "keyserver":"wwwkeys.pgp.net",
+                             "autoCrypto":false,
                              "autoDecrypt":true,
                              "captureWebMail":false,
-                             "useMimeExperimental":false,
-                             "disableSMIMEui":true,
-                             "parseAllHeaders":true,
-                             "show_headers":1,
-                             "hushMailSupport":false,
+                             "composeHtmlAlertCount":3,
+                             "confirmBeforeSend":false,
+                             "disableSMIMEui":false,
+                             "displaySignWarn":true,
+                             "displayPartiallySigned":true,
+                             "doubleDashSeparator":true,
                              "encryptAttachments":1,
+                             "encryptToSelf":true,
+                             "gpgVersionWarnCount":1,
+                             "handleDoubleClick":false,
+                             "hushMailSupport":false,
+                             "initAlertCount":2,
                              "inlineAttachAsciiArmor":false,
                              "inlineAttachExt":".pgp",
-                             "handleDoubleClick":false,
-                             "sendImmediately":true,
-                             "wrapHtmlBeforeSend":true,
+                             "logDirectory":"",
+                             "keepSettingsForReply":true,
+                             "keyserver":"random.sks.keyserver.penguin.de, pgp.dtype.org, keyserver.kjsl.com, ldap://certserver.pgp.com",
+                             "maxIdleMinutes":5,
+                             "mimeHashAlgorithm":1,
+                             "noPassphrase":false,
+                             "parseAllHeaders":true,
+                             "quotedPrintableWarn":0,
+                             "respectHttpProxy":true,
+                             "recipientsSelectionOption":1,
+                             "saveEncrypted":0,
+                             "show_headers":1,
+                             "supportMultiPass":false,
+                             "useDefaultComment":false,
+                             "useMimeExperimental":false,
                              "useGpgAgent":false,
-                             "recipientsSelectionOption":1
+                             "usePGPMimeOption":PGP_MIME_POSSIBLE,
+                             "wrapHtmlBeforeSend":true
                             };
 
 var gEnigLogLevel = 2;     // Output only errors/warnings by default
@@ -138,7 +167,6 @@ try {
 
 function EnigGetFrame(win, frameName) {
   DEBUG_LOG("enigmailCommon.js: EnigGetFrame: name="+frameName+"\n");
-  dump("direct="+win.frames[frameName]+"\n");
   for (var j=0; j<win.frames.length; j++) {
     dump(win.frames[j].name+"\n");
     if (win.frames[j].name == frameName) {
@@ -221,7 +249,7 @@ function GetEnigmailSvc() {
         // Display initialization error alert
         var errMsg = gEnigmailSvc.initializationError ? gEnigmailSvc.initializationError : "Error in initializing Enigmail service";
 
-        errMsg += "\n\nTo avoid this alert, either fix the problem or uninstall Enigmail using the Edit->Preferences->Privacy&Security->Enigmail menu"
+        errMsg += "\n\n"+EnigGetString("avoidInitErr");
 
         EnigAlertCount("initAlertCount", "Enigmail: "+errMsg);
       }
@@ -252,78 +280,32 @@ function GetEnigmailSvc() {
 }
 
 
-function EnigUpdate_0_60() {
-  DEBUG_LOG("enigmailCommon.js: EnigUpdate_0_60: \n");
-
-  var savePrefs = false;
-
+function EnigUpdate_0_80() {
   try {
-    var userIdSource = gPrefEnigmail.getIntPref("userIdSource");
+    var oldVer=EnigGetPref("configuredVersion");
 
-    gPrefEnigmail.deleteBranch("userIdSource");
-    savePrefs = true;
-
-    if (!EnigGetPref("userIdFromAddr")) {
-
-      var userIdFromAddr = false;
-
-      if (userIdSource > 1) {
-        userIdFromAddr = true;
-
-      } else {
-        var userIdValue = EnigGetPref("userIdValue");
-
-        var mesg = EnigGetString("specifyEmail");
-
-        var valueObj = new Object();
-        valueObj.value = userIdValue;
-
-        if (EnigPromptValue(mesg, valueObj)) {
-          userIdValue = valueObj.value;
-          EnigSetPref("userIdValue", userIdValue);
+    if (oldVer.substring(0,1)=="0"){
+      if (oldVer.substring(0,4)<"0.83") {
+        var keySrv = EnigGetPref("keyserver");
+        if (keySrv.indexOf(",") == -1) {
+          var newKeySrv = gEnigmailPrefDefaults.keyserver.replace(keySrv, "");
+          newKeySrv = newKeySrv.replace(/(^, |, $)/, "").replace(/, , /,", ");
+          EnigSetPref("keyserver", keySrv+", "+newKeySrv);
         }
-
-        userIdFromAddr = !userIdValue;
-
-        if (userIdFromAddr)
-          EnigAlert(EnigGetString("usingFrom"));
-        else
-          EnigAlert(EnigGetString("usingId",userIdValue));
-
       }
-
-      EnigSetPref("userIdFromAddr", userIdFromAddr);
+      if (oldVer.substring(0,4)<"0.81") {
+        window.openDialog("chrome://enigmail/content/enigmailUpgrade.xul",
+            "", "dialog,modal,centerscreen");
+      }
     }
-
-  } catch (ex) {}
-
-  try {
-    var defaultEncryptMsg = gPrefEnigmail.getBoolPref("defaultEncryptMsg");
-
-    gPrefEnigmail.deleteBranch("defaultEncryptMsg");
-    savePrefs = true;
-  } catch (ex) {}
-
-  try {
-    var defaultEncryptSignMsg = gPrefEnigmail.getBoolPref("defaultEncryptSignMsg");
-    gPrefEnigmail.deleteBranch("defaultEncryptSignMsg");
-    savePrefs = true;
-
-    if (defaultEncryptSignMsg)
-        gPrefEnigmail.setIntPref("defaultEncryptionOption", 2);
-
-  } catch (ex) {}
-
-  if (savePrefs) {
-    DEBUG_LOG("enigmailCommon.js: EnigUpdate_0_60: Updating prefs\n");
-    EnigSavePrefs();
   }
+  catch (ex) {}
 }
 
 function EnigConfigure() {
   try {
     // Updates for specific versions (to be cleaned-up periodically)
-    EnigUpdate_0_60();
+    EnigUpdate_0_80();
   } catch (ex) {}
 
   var msg = EnigGetString("configNow",gEnigmailVersion);
@@ -358,7 +340,7 @@ function EnigConfigure() {
 
   if (buttonPressed == 0) {
     // Configure now
-    EnigPrefWindow();
+    EnigPrefWindow(true,(navigator.vendor=="Thunderbird" ? "thunderbird" : "seamonkey"));
 
   } else {
     // "Do not ask me again" => "already configured"
@@ -536,14 +518,55 @@ function EnigAlertCount(countPrefName, mesg) {
 }
 
 function EnigConfirm(mesg) {
-  return gEnigPromptSvc.confirm(window, EnigGetString("enigConfirm"), mesg);
+  var dummy=new Object();
+
+  var buttonPressed = gEnigPromptSvc.confirmEx(window,
+                        EnigGetString("enigConfirm"),
+                        mesg,
+                        (gEnigPromptSvc.BUTTON_TITLE_YES * ENIG_BUTTON_POS_0) +
+                        (gEnigPromptSvc.BUTTON_TITLE_NO * ENIG_BUTTON_POS_1),
+                        null, null, null,
+                        null, dummy);
+
+  return (buttonPressed == 0);
+}
+
+function EnigConfirmPref(mesg, prefText) {
+  const notSet = 0;
+  const yes = 1;
+  const no = 2;
+
+  var prefValue = EnigGetPref(prefText);
+  switch (prefValue) {
+  case notSet:
+    var checkBoxObj = { value: false} ;
+    var buttonPressed = gEnigPromptSvc.confirmEx(window,
+                          EnigGetString("enigConfirm"),
+                          mesg,
+                          (gEnigPromptSvc.BUTTON_TITLE_YES * ENIG_BUTTON_POS_0) +
+                          (gEnigPromptSvc.BUTTON_TITLE_NO * ENIG_BUTTON_POS_1),
+                          null, null, null,
+                          EnigGetString("dlgKeepSetting"), checkBoxObj);
+    if (checkBoxObj.value) {
+      EnigSetPref(prefText, (buttonPressed==0 ? yes : no));
+    }
+    return (buttonPressed==0 ? 1 : 0);
+
+  case yes:
+    return 1;
+
+  case no:
+    return 0;
+
+  default:
+    return -1;
+  }
 }
 
 function EnigError(mesg) {
   return gEnigPromptSvc.alert(window, EnigGetString("enigError"), mesg);
 }
 
-
 function EnigPromptValue(mesg, valueObj) {
   var checkObj = new Object();
   return gEnigPromptSvc.prompt(window, EnigGetString("enigPrompt"),
@@ -570,19 +593,21 @@ function EnigOverrideAttribute(elementIdList, attrName, prefix, suffix) {
 }
 
 
-function EnigPrefWindow() {
-  goPreferences("securityItem",
-                "chrome://enigmail/content/pref-enigmail.xul",
-                "enigmail");
+function EnigPrefWindow(showBasic, clientType, selectTab) {
+  window.openDialog("chrome://enigmail/content/pref-enigmail.xul",
+                    "_blank", "chrome,resizable=yes",
+                    {'showBasic': showBasic,
+                     'clientType': clientType,
+                     'selectTab': selectTab});
 }
 
+
 function EnigAdvPrefWindow() {
-  window.openDialog("chrome://enigmail/content/pref-enigmail-adv.xul",
-                    "_blank", "chrome,resizable=yes");
+  EnigAlert("This function doesn't exist anymore!");
 }
 
 function EnigHelpWindow(source) {
-   
+
    var helpUrl = "http://enigmail.mozdev.org/help.html";
 
    if (source)
@@ -732,6 +757,25 @@ function EnigSetPref(prefName, value) {
    return retVal;
 }
 
+function EnigGetSignMsg(identity) {
+  var sign = null;
+
+  if (gEnigPrefRoot.getPrefType("mail.identity."+identity.key+".pgpSignPlain")==0) {
+    if (gEnigPrefRoot.getPrefType("mail.identity."+identity.key+".pgpSignMsg")==0) {
+      sign=identity.getBoolAttribute("pgpAlwaysSign");
+      identity.setBoolAttribute("pgpSignEncrypted", sign);
+      identity.setBoolAttribute("pgpSignPlain", sign);
+    }
+    else {
+      sign = identity.getIntAttribute("pgpSignMsg");
+      identity.setBoolAttribute("pgpSignEncrypted", sign==1);
+      identity.setBoolAttribute("pgpSignPlain", sign>0);
+    }
+    gEnigPrefRoot.deleteBranch("mail.identity."+identity.key+".pgpSignMsg");
+    gEnigPrefRoot.deleteBranch("mail.identity."+identity.key+".pgpAlwaysSign");
+  }
+
+}
 
 function EnigRequestObserver(terminateFunc, terminateArg)
 {
@@ -806,18 +850,6 @@ function EnigGetDeepText(node, findStr) {
 
   DEBUG_LOG("enigmailCommon.js: EnigDeepText: <" + node.tagName + ">, '"+findStr+"'\n");
 
-  /*
-  try {
-    var enigMimeService = Components.classes[ENIG_ENIGMIMESERVICE_CONTRACTID].getService(Components.interfaces.nsIEnigMimeService);
-
-    var plainText = enigMimeService.getPlainText(node, findStr);
-    return plainText;
-
-  } catch (ex) {
-    DEBUG_LOG("enigmailCommon.js: EnigDeepText: Failed to access EnigMimeService\n");
-  }
-  */
-  
   if (findStr) {
     if (node.innerHTML.replace(/&nbsp;/g, " ").indexOf(findStr) < 0) {
       // exit immediately if findStr is not found at all
@@ -836,7 +868,6 @@ function EnigGetDeepText(node, findStr) {
         return "";
      }
   }
-  
 
   return plainText;
 
@@ -991,7 +1022,7 @@ function EnigKeygen() {
 
   window.openDialog('chrome://enigmail/content/enigmailKeygen.xul',
                     'Enigmail Key Generation',
-                    'chrome,dialog,close=no,resizable=yes,width=600');
+                    'chrome,dialog,modal,close=no,resizable=yes,width=600');
 
 }
 
@@ -1074,22 +1105,22 @@ function EnigGetString(aStr) {
 
 // Remove all quoted strings (and angle brackets) from a list of email
 // addresses, returning a list of pure email addresses
-function enigStripEmail(mailAddrs) {
+function EnigStripEmail(mailAddrs) {
 
   var qStart, qEnd;
   while ((qStart = mailAddrs.indexOf('"')) != -1) {
      qEnd = mailAddrs.indexOf('"', qStart+1);
      if (qEnd == -1) {
-       ERROR_LOG("enigmailMsgComposeOverlay.js: enigStripEmail: Unmatched quote in mail address: "+mailAddrs+"\n");
+       ERROR_LOG("enigmailMsgComposeOverlay.js: EnigStripEmail: Unmatched quote in mail address: "+mailAddrs+"\n");
        throw Components.results.NS_ERROR_FAILURE;
      }
-  
+
      mailAddrs = mailAddrs.substring(0,qStart) + mailAddrs.substring(qEnd+1);
   }
-  
+
   // Eliminate all whitespace, just to be safe
   mailAddrs = mailAddrs.replace(/\s+/g,"");
-  
+
   // Extract pure e-mail address list (stripping out angle brackets)
   mailAddrs = mailAddrs.replace(/(^|,)[^,]*<([^>]+)>[^,]*/g,"$1$2");
 
@@ -1120,3 +1151,162 @@ function EnigGetTempDir() {
   }
   return tmpDir;
 }
+
+
+function EnigDisplayPrefs(showDefault, showPrefs, setPrefs) {
+   DEBUG_LOG("enigmailCommon.js: EnigDisplayPrefs\n");
+
+   for (var prefName in gEnigmailPrefDefaults) {
+      var prefElement = document.getElementById("enigmail_"+prefName);
+
+      if (prefElement) {
+         var defaultValue = gEnigmailPrefDefaults[prefName];
+         var prefValue = showDefault ? defaultValue : EnigGetPref(prefName);
+
+         DEBUG_LOG("enigmailCommon.js: EnigDisplayPrefs: "+prefName+"="+prefValue+"\n");
+
+         switch (typeof defaultValue) {
+         case "boolean":
+            if (showPrefs) {
+               if (prefValue) {
+                  prefElement.setAttribute("checked", "true");
+               } else {
+                  prefElement.removeAttribute("checked");
+               }
+            }
+
+            if (setPrefs) {
+               if (prefElement.checked) {
+                  EnigSetPref(prefName, true);
+               } else {
+                  EnigSetPref(prefName, false);
+               }
+            }
+
+         break;
+
+         case "number":
+            if (showPrefs)
+              prefElement.value = prefValue;
+
+            if (setPrefs) {
+               try {
+                 EnigSetPref(prefName, 0+prefElement.value);
+               } catch (ex) {
+               }
+            }
+         break;
+
+         case "string":
+            if (showPrefs)
+              prefElement.value = prefValue;
+            if (setPrefs)
+              EnigSetPref(prefName, prefElement.value);
+            break;
+
+         default:
+         }
+      }
+   }
+}
+
+
+function EnigReceiveKey(parent, msgParentWindow, recvFlags, keyId,
+                        progressBar, requestObserver,
+                        errorMsgObj) {
+
+  if (keyId) {
+
+    var valueObj = { keyId: keyId };
+    var checkObj = new Object();
+
+    window.openDialog("chrome://enigmail/content/enigmailKeyserverDlg.xul",
+          "", "dialog,modal,centerscreen", valueObj, checkObj);
+
+    if (! checkObj.value) {
+      errorMsgObj.value = EnigGetString("failCancel");
+      return null;
+    }
+
+    var keyserver = checkObj.value;
+  }
+  else {
+    return null;
+  }
+
+  var enigmailSvc = GetEnigmailSvc();
+  if (!enigmailSvc)
+     return null;
+
+  if (progressBar) {
+    // wait one second before displaying the progress bar
+    var progressParam=Components.classes["@mozilla.org/messengercompose/composeprogressparameters;1"].createInstance(Components.interfaces.nsIMsgComposeProgressParams);
+    parent.setTimeout(progressBar.openProgressDialog, 1000, parent, msgWindow, "chrome://enigmail/content/enigRetrieveProgress.xul", progressParam);
+    //progressBar.openProgressDialog(parent, msgWindow, "chrome://enigmail/content/enigRetrieveProgress.xul", progressParam);
+    progressBar.onStateChange(null, null, Components.interfaces.nsIWebProgressListener.STATE_START, 0);
+  }
+
+  return enigmailSvc.receiveKey(keyserver, keyId, requestObserver, errorMsgObj);
+}
+
+
+function EnigUninstall() {
+  const JSLIB_MIN_VER="0.1.128";
+  if ((typeof(JS_LIB_VERSION) != "string") || (JS_LIB_VERSION < JSLIB_MIN_VER)) {
+    EnigAlert(EnigGetString("jslibNeeded", JSLIB_MIN_VER));
+    return;
+  }
+  window.close();
+  if (!EnigConfirm(EnigGetString("uninstallConfirm")))
+    return;
+
+  window.openDialog("chrome://enigmail/content/enigmailUninstall.xul",
+          "", "dialog,modal,centerscreen", {performUninst: true});
+
+  return;
+}
+
+function EnigFilePicker(title, displayDir, save, defaultExtension, defaultName, filterPairs) {
+  DEBUG_LOG("enigmailCommon.js: EnigFilePicker: "+save+"\n");
+
+  const nsIFilePicker = Components.interfaces.nsIFilePicker;
+  var filePicker = Components.classes["@mozilla.org/filepicker;1"].createInstance();
+  filePicker = filePicker.QueryInterface(nsIFilePicker);
+
+  var mode = save ? nsIFilePicker.modeSave : nsIFilePicker.modeOpen;
+
+  filePicker.init(window, title, mode);
+
+  if (displayDir) {
+    var localFile = Components.classes[ENIG_LOCAL_FILE_CONTRACTID].createInstance(Components.interfaces.nsILocalFile);
+
+    try {
+      localFile.initWithPath(displayDir);
+      filePicker.displayDirectory = localFile;
+    } catch (ex) {
+    }
+  }
+
+  if (defaultExtension)
+    filePicker.defaultExtension = defaultExtension;
+
+  if (defaultName)
+    filePicker.defaultString=defaultName;
+
+  var nfilters = 0;
+  if (filterPairs && filterPairs.length)
+    nfilters = filterPairs.length / 2;
+
+  for (var index=0; index < nfilters; index++) {
+    filePicker.appendFilter(filterPairs[2*index], filterPairs[2*index+1]);
+  }
+
+  filePicker.appendFilters(nsIFilePicker.filterAll);
+
+  if (filePicker.show() == nsIFilePicker.returnCancel)
+    return null;
+
+  var file = filePicker.file.QueryInterface(Components.interfaces.nsILocalFile);
+
+  return file;
+}
