@@ -1,3 +1,36 @@
+/*
+The contents of this file are subject to the Mozilla Public
+License Version 1.1 (the "MPL"); you may not use this file
+except in compliance with the MPL. You may obtain a copy of
+the MPL at http://www.mozilla.org/MPL/
+
+Software distributed under the MPL is distributed on an "AS
+IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
+implied. See the MPL for the specific language governing
+rights and limitations under the MPL.
+
+The Original Code is Enigmail.
+
+The Initial Developer of the Original Code is Ramalingam Saravanan.
+Portions created by Ramalingam Saravanan <svn@xmlterm.org> are
+Copyright (C) 2001 Ramalingam Saravanan. All Rights Reserved.
+
+Contributor(s):
+Patrick Brunschwig <patrick.brunschwig@gmx.net>
+
+Alternatively, the contents of this file may be used under the
+terms of the GNU General Public License (the "GPL"), in which case
+the provisions of the GPL are applicable instead of
+those above. If you wish to allow use of your version of this
+file only under the terms of the GPL and not to allow
+others to use your version of this file under the MPL, indicate
+your decision by deleting the provisions above and replace them
+with the notice and other provisions required by the GPL.
+If you do not delete the provisions above, a recipient
+may use your version of this file under either the MPL or the
+GPL.
+*/
+
 // Uses: chrome://enigmail/content/enigmailCommon.js
 
 // Initialize enigmailCommon
@@ -11,6 +44,9 @@ var gEnigLastSaveDir = "";
 
 var gEnigMessagePane = null;
 var gEnigNoShowReload = false;
+var gEnigLastEncryptedURI = null;
+var gEnigDecryptButton = null;
+var gEnigIpcRequest = null;
 
 var gEnigRemoveListener = false;
 
@@ -29,12 +65,12 @@ window.addEventListener("unload", enigMessengerFinish,  false);
 function enigMessengerStartup() {
   DEBUG_LOG("enigmailMessengerOverlay.js: Startup\n");
 
-  enigUpdateOptionsDisplay();
+  // enigUpdateOptionsDisplay();
 
   // Override SMIME ui
   var smimeStatusElement = document.getElementById("cmd_viewSecurityStatus");
   if (smimeStatusElement) {
-    smimeStatusElement.setAttribute("oncommand", "enigViewSecurityInfo();");
+    smimeStatusElement.setAttribute("oncommand", "enigViewSecurityInfo(null, true);");
   }
 
   // Override print command
@@ -75,18 +111,19 @@ function enigMessengerStartup() {
   EnigOverrideAttribute( viewElementIds, "oncommand",
                          "enigMsgViewHeaders('", "');");
 
+  enigOverrideLayoutChange();
+
   EnigShowHeadersAll(true);
   gEnigSavedHeaders = null;
 
+  gEnigDecryptButton = document.getElementById("button-enigmail-decrypt");
   gEnigMessagePane = document.getElementById("messagepane");
+  enigMessageFrameLoad();
 
   // Need to add event listener to gEnigMessagePane to make it work
   // Adding to msgFrame doesn't seem to work
   gEnigMessagePane.addEventListener("unload", enigMessageFrameUnload, true);
-
-  // Commented out; clean-up now handled by HdrView and Unload
-  //var tree = GetThreadTree();
-  //tree.addEventListener("click", enigThreadPaneOnClick, true);
+  gEnigMessagePane.addEventListener("load", enigMessageFrameLoad, true);
 
   if (EnigGetPref("handleDoubleClick")) {
     // ovveride function for double clicking an attachment
@@ -100,9 +137,13 @@ function enigMessengerFinish() {
   DEBUG_LOG("enigmailMessengerOverlay.js: Finish\n");
 }
 
-function enigViewSecurityInfo() {
+
+function enigViewSecurityInfo(event, displaySmimeMsg) {
   DEBUG_LOG("enigmailMessengerOverlay.js: enigViewSecurityInfo\n");
 
+  if (event && event.button != 0)
+    return;
+
   if (gEnigSecurityInfo) {
     // Display OpenPGP security info
 
@@ -111,34 +152,14 @@ function enigViewSecurityInfo() {
     if (keyserver && gEnigSecurityInfo.keyId &&
         (gEnigSecurityInfo.statusFlags & nsIEnigmail.UNVERIFIED_SIGNATURE) ) {
 
-      var pubKeyId = "0x" + gEnigSecurityInfo.keyId.substr(8, 8);
-
-      var mesg =  gEnigSecurityInfo.statusInfo + EnigGetString("keyImport",pubKeyId);
-
-      if (EnigConfirm(mesg)) {
-        var recvErrorMsgObj = new Object();
-        var recvFlags = nsIEnigmail.UI_INTERACTIVE;
-
-        var enigmailSvc = GetEnigmailSvc();
-        var exitStatus = enigmailSvc.receiveKey(window, recvFlags, pubKeyId,
-                                                recvErrorMsgObj);
-
-        if (exitStatus == 0) {
-          enigMessageReload(false);
-        } else {
-          EnigAlert(EnigGetString("keyImportError")+recvErrorMsgObj.value);
-        }
-      }
+        enigHandleUnknownKey();
 
     } else if ( (gEnigSecurityInfo.statusFlags & nsIEnigmail.NODATA) &&
          (gEnigSecurityInfo.statusFlags &
            (nsIEnigmail.PGP_MIME_SIGNED | nsIEnigmail.PGP_MIME_ENCRYPTED)) ) {
 
       if (typeof(ReloadWithAllParts) == "function") {
-
-        var mesg = EnigGetString("reloadImapMessage");
-
-        if (EnigConfirm(mesg))
+        if (EnigConfirm(EnigGetString("reloadImapMessage")))
           ReloadWithAllParts();
 
       } else {
@@ -151,7 +172,12 @@ function enigViewSecurityInfo() {
 
   } else {
     // Display SMIME security info
-    showMessageReadSecurityInfo();
+    if (displaySmimeMsg) {
+      showMessageReadSecurityInfo();
+    }
+    else {
+      EnigAlert(EnigGetString("noPgpMessage"));
+    }
   }
 }
 
@@ -173,7 +199,7 @@ function enigInitViewHeadersMenu() {
   case 2:
     id = "viewallheaders";
     break;
-  case 1:	
+  case 1:
   default:
     id = "viewnormalheaders";
     break;
@@ -181,7 +207,7 @@ function enigInitViewHeadersMenu() {
 
   var menuitem = document.getElementById(id);
   if (menuitem)
-    menuitem.setAttribute("checked", "true"); 
+    menuitem.setAttribute("checked", "true");
 }
 
 function enigMsgViewHeaders(elementId) {
@@ -224,7 +250,7 @@ function enigMessageCleanup() {
     var statusText = document.getElementById("expandedEnigmailStatusText");
 
     if (statusText)
-      statusText.setAttribute("value", "");
+      statusText.firstChild.data="";
   }
 
   if (gEnigCreatedURIs.length) {
@@ -243,11 +269,6 @@ function enigMessageCleanup() {
   gEnigSecurityInfo = null;
 }
 
-function enigMessageUnload() {
-  DEBUG_LOG("enigmailMessengerOverlay.js: MessageUnload\n");
-}
-
-
 function enigMimeInit() {
   DEBUG_LOG("enigmailMessengerOverlay.js: *****enigMimeInit\n");
 
@@ -294,6 +315,24 @@ function enigMimeInit() {
 
 function enigMessageFrameLoad() {
   DEBUG_LOG("enigmailMessengerOverlay.js: enigMessageFrameLoad\n");
+  if (gEnigDecryptButton) {
+    var setDisable=false;
+    try {
+      if (gDBView) {
+        var uriType="none";
+        try {
+          uriType=typeof gDBView.URIForFirstSelectedMessage;
+        }
+        catch (ex) {}
+        setDisable=((gDBView.numSelected > 1) || (uriType!="string"));
+      }
+      else {
+        setDisable=true;
+      }
+    }
+    catch (ex) {}
+    gEnigDecryptButton.disabled=setDisable;
+  }
 }
 
 function enigMessageFrameUnload() {
@@ -315,6 +354,36 @@ function enigThreadPaneOnClick() {
   DEBUG_LOG("enigmailMessengerOverlay.js: enigThreadPaneOnClick\n");
 }
 
+function enigOverrideLayoutChange() {
+  DEBUG_LOG("enigmailMessengerOverlay.js: enigOverrideLayoutChange\n");
+  var viewTypeElementIds = ["messagePaneVertical",
+                            "messagePaneClassic",
+                            "messagePaneWide"];
+  for (var i = 0; i < viewTypeElementIds.length; i++) {
+    var elementId = viewTypeElementIds[i];
+    var element = document.getElementById(elementId);
+    if (element) {
+      try {
+        var oldValue = element.getAttribute("oncommand");
+        var arg=oldValue.replace(/^(.*)(\(.*\))/, "$2");
+        element.setAttribute("oncommand", "enigChangeMailLayout"+arg);
+      } catch (ex) {}
+    } else {
+      // we can break if the vertical Thunderbird pane
+      // doesn't exist, because that means it's Seamonkey!
+      break;
+    }
+  }
+}
+
+function enigChangeMailLayout(viewType) {
+  ChangeMailLayout(viewType);
+
+  // This event requires that we re-subscribe to these events!
+  gEnigMessagePane.addEventListener("unload", enigMessageFrameUnload, true);
+  gEnigMessagePane.addEventListener("load", enigMessageFrameLoad, true);
+}
+
 function enigGetCurrentMsgUriSpec() {
   try {
     if (GetNumSelectedMessages() != 1)
@@ -357,12 +426,23 @@ function enigGetCurrentMsgUrl() {
 
 function enigUpdateOptionsDisplay() {
   DEBUG_LOG("enigmailMessengerOverlay.js: enigUpdateOptionsDisplay: \n");
-   var optList = ["autoDecrypt"];
+  var optList = ["autoDecrypt"];
 
-   for (var j=0; j<optList.length; j++) {
-     var menuElement = document.getElementById("enigmail_"+optList[j]);
-     menuElement.setAttribute("checked", EnigGetPref(optList[j]) ? "true" : "false");
-   }
+  for (var j=0; j<optList.length; j++) {
+    var menuElement = document.getElementById("enigmail_"+optList[j]);
+    menuElement.setAttribute("checked", EnigGetPref(optList[j]) ? "true" : "false");
+  }
+
+  optList = ["decryptverify", "importpublickey", "savedecrypted"];
+  for (j=0; j<optList.length; j++) {
+    var menuElement = document.getElementById("enigmail_"+optList[j]);
+    if (gEnigDecryptButton && gEnigDecryptButton.disabled) {
+       menuElement.setAttribute("disabled", "true");
+    }
+    else {
+       menuElement.removeAttribute("disabled");
+    }
+  }
 }
 
 
@@ -416,11 +496,15 @@ function enigMessageDecrypt(event) {
       }
 
       var emailAttachment = false;
+      var embeddedSigned = null;
       for (var indexb in currentAttachments) {
         var attachment = currentAttachments[indexb];
-        if (attachment.contentType.search(/^message\/rfc822(;|$)/i)  == 0) {
+        if (attachment.contentType.search(/^message\/rfc822(;|$)/i) == 0) {
           emailAttachment = true;
         }
+        if (attachment.contentType.search(/^application\/pgp-signature/i) == 0) {
+          embeddedSigned = attachment.url.replace(/\.\d+\.\d+$/, "");
+        }
         DEBUG_LOG("enigmailMessengerOverlay.js: "+indexb+": "+attachment.contentType+"\n");
         //DEBUG_LOG("enigmailMessengerOverlay.js: "+indexb+": "+attachment.url+"\n");
       }
@@ -471,12 +555,28 @@ function enigMessageDecrypt(event) {
       var msgFrame = EnigGetFrame(window, "messagepane");
       msgFrame.location = "enigmail:dummy";
 
-      return;
+    }
+    return;
+  }
+
+  var tryVerify = false;
+  var enableSubpartTreatment = false;
+  // special treatment for embedded signed messages
+  if (embeddedSigned) {
+    if (contentType.search(/^multipart\/encrypted(;|$)/i) == 0)
+      tryVerify = true;
+
+    if (contentType.search(/^multipart\/mixed(;|$)/i) == 0) {
+      tryVerify = true;
+      enableSubpartTreatment = true;
     }
   }
 
-  if ( (contentType.search(/^multipart\/signed(;|$)/i) == 0) &&
-       (contentType.search(/application\/pgp-signature/i) >= 0) ) {
+  if ((contentType.search(/^multipart\/signed(;|$)/i) == 0) &&
+       (contentType.search(/application\/pgp-signature/i) >= 0))
+    tryVerify=true;
+
+  if (tryVerify) {
     // multipart/signed
     DEBUG_LOG("enigmailMessengerOverlay.js: multipart/signed\n");
 
@@ -486,12 +586,16 @@ function enigMessageDecrypt(event) {
 
     var msgUriSpec = enigGetCurrentMsgUriSpec();
     var mailNewsUrl = enigGetCurrentMsgUrl();
+    if (embeddedSigned) {
+      mailNewsUrl.spec = embeddedSigned;
+    }
 
     if (mailNewsUrl) {
       const ENIG_ENIGMIMEVERIFY_CONTRACTID = "@mozilla.org/enigmail/mime-verify;1";
       var verifier = Components.classes[ENIG_ENIGMIMEVERIFY_CONTRACTID].createInstance(Components.interfaces.nsIEnigMimeVerify);
 
-      verifier.init(mailNewsUrl, msgWindow, msgUriSpec, true);
+      verifier.init(mailNewsUrl, msgWindow, msgUriSpec,
+                    true, enableSubpartTreatment);
 
       return;
     }
@@ -511,7 +615,7 @@ function enigMessageParse(interactive, importOnly, contentEncoding) {
   var bodyElement = msgFrame.document.getElementsByTagName("body")[0];
   DEBUG_LOG("enigmailMessengerOverlay.js: bodyElement="+bodyElement+"\n");
 
-  var findStr = interactive ? "" : "-----BEGIN PGP";
+  var findStr = interactive ? null : "-----BEGIN PGP";
   var msgText = EnigGetDeepText(bodyElement, findStr);
 
   if (!msgText) {
@@ -536,7 +640,7 @@ function enigMessageParse(interactive, importOnly, contentEncoding) {
       tail=msgText.substring(endStart+nextLine).replace(/^[\n\r\s]*/,"");
     }
   }
-  
+
   //DEBUG_LOG("enigmailMessengerOverlay.js: msgText='"+msgText+"'\n");
 
   var mailNewsUrl = enigGetCurrentMsgUrl();
@@ -553,6 +657,9 @@ function enigMessageParseCallback(msgText, contentEncoding, charset, interactive
                                   head, tail) {
   DEBUG_LOG("enigmailMessengerOverlay.js: enigMessageParseCallback: "+interactive+", "+interactive+", importOnly="+importOnly+", charset="+charset+", msgUrl="+messageUrl+", retry="+retry+", signature='"+signature+"'\n");
 
+  if (!msgText)
+    return;
+
   var enigmailSvc = GetEnigmailSvc();
   if (!enigmailSvc)
     return;
@@ -561,6 +668,7 @@ function enigMessageParseCallback(msgText, contentEncoding, charset, interactive
   var exitCode;
   var newSignature = "";
   var statusFlags = 0;
+
   var errorMsgObj = new Object();
   var keyIdObj    = new Object();
 
@@ -637,7 +745,7 @@ function enigMessageParseCallback(msgText, contentEncoding, charset, interactive
     if (retry) {
       // Try to verify signature by accessing raw message text directly
       // (avoid recursion by setting retry parameter to false on callback)
-      enigMsgDirect(interactive, importOnly, contentEncoding, charset, newSignature, head, tail);
+      enigMsgDirect(interactive, importOnly, contentEncoding, charset, newSignature, 0, head, tail);
       return;
     }
   }
@@ -690,7 +798,7 @@ function enigMessageParseCallback(msgText, contentEncoding, charset, interactive
       // print a warning if the signed or encrypted part doesn't start
       // quite early in the message
       var matches=head.match(/(\n)/g);
-      if (matches.length >10) {
+      if (matches && matches.length >10) {
         msgRfc822Text=EnigGetString("notePartEncrypted")+"\n\n";
       }
       msgRfc822Text+=head+"\n\n";
@@ -894,7 +1002,7 @@ function enigGetDecryptedMessage(contentType, includeHeaders) {
 
   var contentData = "";
 
-  var headerName, headerValue;
+  var headerName;
 
   if (contentType == "message/rfc822") {
     // message/rfc822
@@ -918,6 +1026,19 @@ function enigGetDecryptedMessage(contentType, includeHeaders) {
                         "Date":    headerList.date,
                         "Newsgroups": msgData.messageHeader.folder.name };
           }
+          else if(urlObj.value.scheme=="imap") {
+            var msgKey=new Object();
+            msgData = urlObj.value.QueryInterface(Components.interfaces.nsIImapUrl);
+            // this only succeeds if exactly one message was selected, which is
+            // at this moment the case
+            msgData.createListOfMessageIdsString(msgKey);
+            var imapMsgHeader = msgData.imapMessageSink.GetMessageHeader(msgKey.value);
+            msgHdr = { "From":    imapMsgHeader.author,
+                        "Subject": imapMsgHeader.subject,
+                        "Cc":      imapMsgHeader.ccList,
+                        "To":      imapMsgHeader.recipients,
+                        "Date":    headerList.date };
+          }
           else {
             msgData = urlObj.value.QueryInterface(Components.interfaces.nsIMailboxUrl);
             msgHdr = { "From":    msgData.messageHeader.author,
@@ -1006,15 +1127,6 @@ function enigGetDecryptedMessage(contentType, includeHeaders) {
   return contentData;
 }
 
-function enigMsgDefaultPrint(elementId) {
-  DEBUG_LOG("enigmailMessengerOverlay.js: enigMsgDefaultPrint: "+elementId+"\n");
-
-  // Reset mail.show_headers pref to "original" value
-  EnigShowHeadersAll(false);
-
-  goDoCommand(elementId.indexOf("printpreview")>=0 ? "cmd_printpreview" : "cmd_print");
-}
-
 function enigMsgForward(elementId, event) {
   DEBUG_LOG("enigmailMessengerOverlay.js: enigMsgForward: "+elementId+", "+event+"\n");
 
@@ -1033,6 +1145,15 @@ function enigMsgForward(elementId, event) {
   }
 }
 
+function enigMsgDefaultPrint(elementId) {
+  DEBUG_LOG("enigmailMessengerOverlay.js: enigMsgDefaultPrint: "+elementId+"\n");
+
+  // Reset mail.show_headers pref to "original" value
+  EnigShowHeadersAll(false);
+
+  goDoCommand(elementId.indexOf("printpreview")>=0 ? "cmd_printpreview" : "cmd_print");
+}
+
 function enigMsgPrint(elementId) {
   DEBUG_LOG("enigmailMessengerOverlay.js: enigMsgPrint: "+elementId+"\n");
 
@@ -1070,7 +1191,6 @@ function enigMsgPrint(elementId) {
   DEBUG_LOG("enigmailMessengerOverlay.js: enigMsgPrint: uri="+uri+"\n");
 
   var messageList = [uri];
-  var numMessages = messageList.length;
 
   if (gPrintSettings == null) {
     gPrintSettings = GetPrintSettings();
@@ -1079,11 +1199,13 @@ function enigMsgPrint(elementId) {
   var printPreview = (elementId.indexOf("printpreview")>=0);
   var printEngineWindow = window.openDialog("chrome://messenger/content/msgPrintEngine.xul",
                                         "",
-                                        "chrome,dialog=no,all",
-                                        numMessages, messageList, statusFeedback, gPrintSettings,
-                                        printPreview);
+                                        "chrome,dialog=no,all,centerscreen",
+                                        1, messageList, statusFeedback, gPrintSettings,
+                                        printPreview, Components.interfaces.nsIMsgPrintEngine.MNAB_PRINTPREVIEW_MSG,
+                                        window);
 
   return true;
+
 }
 
 function enigMessageSave() {
@@ -1119,8 +1241,8 @@ function enigMessageSave() {
 
   var textContent = enigGetDecryptedMessage("text/plain", true);
 
-  EnigAlert(textContent);
-  
+//  EnigAlert(textContent);
+
   if (!EnigWriteFileContents(saveFile.path, textContent, null)) {
     EnigAlert("Error in saving to file "+saveFile.path);
     return;
@@ -1129,54 +1251,7 @@ function enigMessageSave() {
   return;
 }
 
-
-function EnigFilePicker(title, displayDir, save, defaultExtension, defaultName, filterPairs) {
-  DEBUG_LOG("enigmailMessengerOverlay.js: EnigFilePicker: "+save+"\n");
-
-  const nsIFilePicker = Components.interfaces.nsIFilePicker;
-  var filePicker = Components.classes["@mozilla.org/filepicker;1"].createInstance();
-  filePicker = filePicker.QueryInterface(nsIFilePicker);
-
-  var mode = save ? nsIFilePicker.modeSave : nsIFilePicker.modeOpen;
-
-  filePicker.init(window, title, mode);
-
-  if (displayDir) {
-    var localFile = Components.classes[ENIG_LOCAL_FILE_CONTRACTID].createInstance(Components.interfaces.nsILocalFile);
-
-    try {
-      localFile.initWithPath(displayDir);
-      filePicker.displayDirectory = localFile;
-    } catch (ex) {
-    }
-  }
-
-  if (defaultExtension)
-    filePicker.defaultExtension = defaultExtension;
-
-  if (defaultName)
-    filePicker.defaultString=defaultName;
-
-  var nfilters = 0;
-  if (filterPairs && filterPairs.length)
-    nfilters = filterPairs.length / 2;
-
-  for (var index=0; index < nfilters; index++) {
-    filePicker.appendFilter(filterPairs[2*index], filterPairs[2*index+1]);
-  }
-
-  filePicker.appendFilters(nsIFilePicker.filterAll);
-
-  if (filePicker.show() == nsIFilePicker.returnCancel)
-    return null;
-
-  var file = filePicker.file.QueryInterface(Components.interfaces.nsILocalFile);
-
-  return file;
-}
-
-
-function enigMsgDirect(interactive, importOnly, contentEncoding, charset, signature, head, tail) {
+function enigMsgDirect(interactive, importOnly, contentEncoding, charset, signature, bufferSize, head, tail) {
   WRITE_LOG("enigmailMessengerOverlay.js: enigMsgDirect: contentEncoding="+contentEncoding+", signature="+signature+"\n");
   var mailNewsUrl = enigGetCurrentMsgUrl();
   if (!mailNewsUrl)
@@ -1184,7 +1259,12 @@ function enigMsgDirect(interactive, importOnly, contentEncoding, charset, signat
 
   var ipcBuffer = Components.classes[ENIG_IPCBUFFER_CONTRACTID].createInstance(Components.interfaces.nsIIPCBuffer);
 
-  ipcBuffer.open(ENIG_MSG_BUFFER_SIZE, false);
+  if (bufferSize > 0) {
+    ipcBuffer.open(bufferSize, false);
+  }
+  else {
+    ipcBuffer.open(ENIG_MSG_BUFFER_SIZE, false);
+  }
 
   var callbackArg = { interactive:interactive,
                       importOnly:importOnly,
@@ -1193,6 +1273,7 @@ function enigMsgDirect(interactive, importOnly, contentEncoding, charset, signat
                       messageUrl:mailNewsUrl.spec,
                       signature:signature,
                       ipcBuffer:ipcBuffer,
+                      expectedBufferSize: bufferSize,
                       head:head,
                       tail:tail };
 
@@ -1234,6 +1315,7 @@ function enigMsgDirectCallback(callbackArg, ctxt) {
 
   var mailNewsUrl = enigGetCurrentMsgUrl();
   var urlSpec = mailNewsUrl ? mailNewsUrl.spec : "";
+  var newBufferSize = 0;
 
   if (urlSpec != callbackArg.messageUrl) {
     ERROR_LOG("enigmailMessengerOverlay.js: enigMsgDirectCallback: Message URL mismatch "+mailNewsUrl.spec+" vs. "+callbackArg.messageUrl+"\n");
@@ -1242,12 +1324,28 @@ function enigMsgDirectCallback(callbackArg, ctxt) {
 
   if (callbackArg.ipcBuffer.overflowed) {
     WARNING_LOG("enigmailMessengerOverlay.js: enigMsgDirectCallback: MESSAGE BUFFER OVERFLOW\n");
+    if (! callbackArg.expectedBufferSize) {
+      // set correct buffer size
+      newBufferSize=((callbackArg.ipcBuffer.totalBytes+1500)/1024).toFixed(0)*1024;
+    }
   }
 
   var msgText = callbackArg.ipcBuffer.getData();
 
   callbackArg.ipcBuffer.shutdown();
 
+  if (newBufferSize > 0) {
+    // retry with correct buffer size
+    enigMsgDirect(callbackArg.interactive,
+                  callbackArg.importOnly,
+                  callbackArg.contentEncoding,
+                  callbackArg.charset,
+                  callbackArg.signature,
+                  newBufferSize,
+                  callbackArg.head,
+                  callbackArg.tail);
+
+  }
   DEBUG_LOG("enigmailMessengerOverlay.js: enigMsgDirectCallback: msgText='"+msgText+"'\n");
 
   enigMessageParseCallback(msgText, callbackArg.contentEncoding,
@@ -1548,28 +1646,57 @@ function enigDecryptAttachmentCallback(callbackArg, ctxt) {
 
         var mimeService = Components.classes[ENIG_MIME_CONTRACTID].getService(Components.interfaces.nsIMIMEService);
         var fileMimeType = mimeService.GetTypeFromFile(outFile);
-        var fileMimeInfo = mimeService.GetFromMIMEType(fileMimeType);
-        if ((fileMimeInfo.preferredAction == fileMimeInfo.useHelperApp ||
-            fileMimeInfo.preferredAction == fileMimeInfo.useSystemDefault) &&
-            (! fileMimeType.indexOf("text/")==0)) {
-          // open attachment with a helper application
-          window.OpenURL(outFileUri.asciiSpec);
-          return;
+        var fileMimeInfo = mimeService.GetFromTypeAndExtension(fileMimeType, fileExt);
+        if (gEnigDecryptButton && gEnigDecryptButton.getAttribute("buttontype")=="seamonkey") {
+          // Seamonkey variant
+          if ((fileMimeInfo.preferredAction == fileMimeInfo.useHelperApp ||
+              fileMimeInfo.preferredAction == fileMimeInfo.useSystemDefault) &&
+              (! fileMimeType.indexOf("text/")==0)) {
+            // open attachment with a helper application
+            window.OpenURL(outFileUri.asciiSpec);
+            return;
+          }
+        }
+        else {
+          // Thunderbird variant
+          if (fileMimeType.indexOf("text/")==0 || fileMimeType.indexOf("image/")==0) {
+            // open attachment using a browser window
+            var w = window.open(outFileUri.asciiSpec,
+                        "_blank",
+                        "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar,width=600,height=400");
+            w.title=outFile.leafName;
+            if (fileMimeType.indexOf("text/")==0)
+                w.document.designMode="on";
+            return;
+          }
+          else {
+            // open attachment with a helper application
+            messenger.OpenURL(outFileUri.asciiSpec);
+            return;
+          }
         }
       }
       catch (ex) {
         // if the attachment file type is unknown, an exception is thrown,
         // so let it be handled by a browser window
-        EnigLoadURLInNavigatorWindow(outFileUri.asciiSpec, true)
+        enigLoadExternalURL(outFileUri.asciiSpec);
         return;
       }
     }
 
-    // open the attachment in a browser window
-    EnigLoadURLInNavigatorWindow(outFileUri.asciiSpec, true)
+    // open the attachment using an external application
+    enigLoadExternalURL(outFileUri.asciiSpec);
   }
 }
 
+function enigLoadExternalURL(url) {
+  if (gEnigDecryptButton && gEnigDecryptButton.getAttribute("buttontype")=="seamonkey") {
+    EnigLoadURLInNavigatorWindow(url, true);
+  }
+  else {
+    messenger.launchExternalURL(url);
+  }
+}
 
 // try to import the PGP keys contained in an attachment
 function enigImportAttachment(argumentsObj) {
@@ -1644,3 +1771,116 @@ function enigAttachmentListClick (elementId, event) {
     attachmentListClick(event);
   }
 }
+
+// download keys
+function enigHandleUnknownKey() {
+  var pubKeyId = "0x" + gEnigSecurityInfo.keyId.substr(8, 8);
+
+  var mesg =  gEnigSecurityInfo.statusInfo + EnigGetString("keyImport",pubKeyId);
+
+  if (EnigConfirm(mesg)) {
+    var recvErrorMsgObj = new Object();
+    var recvFlags = nsIEnigmail.UI_INTERACTIVE;
+    var progressBar=Components.classes["@mozilla.org/messenger/progress;1"].createInstance(Components.interfaces.nsIMsgProgress);
+    var requestObserver = new EnigRequestObserver(enigReceiveKeyTerminate, {'progressBar': progressBar, 'callType': 1});
+
+    var progressListener = {
+        onStateChange: function(aWebProgress, aRequest, aStateFlags, aStatus)
+        {
+          if (aStateFlags & Components.interfaces.nsIWebProgressListener.STATE_STOP) {
+            if (progressBar.processCanceledByUser)
+              enigReceiveKeyCancel(progressBar);
+          }
+        },
+
+        onProgressChange: function(aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress)
+        {},
+
+        onLocationChange: function(aWebProgress, aRequest, aLocation)
+        {},
+
+        onStatusChange: function(aWebProgress, aRequest, aStatus, aMessage)
+        {},
+
+        onSecurityChange: function(aWebProgress, aRequest, state)
+        {},
+
+        QueryInterface : function(iid)
+        {
+          if (iid.equals(Components.interfaces.nsIWebProgressListener) ||
+              iid.equals(Components.interfaces.nsISupportsWeakReference) ||
+              iid.equals(Components.interfaces.nsISupports))
+            return this;
+
+          throw Components.results.NS_NOINTERFACE;
+        }
+    };
+    progressBar.registerListener(progressListener);
+
+    var ipcRequest = EnigReceiveKey(window, msgWindow,
+                      recvFlags, pubKeyId,
+                      progressBar,
+                      requestObserver,
+                      recvErrorMsgObj);
+
+    if (ipcRequest == null) {
+      EnigAlert(EnigGetString("keyImportError")+recvErrorMsgObj.value);
+    }
+    gEnigIpcRequest = ipcRequest;
+  }
+}
+
+function enigReceiveKeyTerminate (terminateArg, ipcRequest) {
+  DEBUG_LOG("enigmailMessengerOverlay.js: receiveKeyTerminate\n");
+
+  if (terminateArg && terminateArg.progressBar) {
+    terminateArg.progressBar.onStateChange(null, null, Components.interfaces.nsIWebProgressListener.STATE_STOP, 0);
+  }
+
+  var keyRetrProcess = gEnigIpcRequest.pipeTransport;
+  var exitCode;
+
+  if (keyRetrProcess && !keyRetrProcess.isAttached()) {
+    keyRetrProcess.terminate();
+    exitCode = keyRetrProcess.exitCode();
+    DEBUG_LOG("enigmailMessengerOverlay.js: enigReceiveKeyTerminate: exitCode = "+exitCode+"\n");
+  }
+
+  if (exitCode==0) {
+    enigMessageReload(false);
+  }
+  else {
+    var errorMsg="";
+    try {
+      var keygenConsole = gEnigIpcRequest.stderrConsole.QueryInterface(Components.interfaces.nsIPipeConsole);
+
+      if (keygenConsole && keygenConsole.hasNewData()) {
+        errorMsg = keygenConsole.getData();
+        var enigmailSvc = GetEnigmailSvc();
+        if (enigmailSvc) {
+          var statusFlagsObj=new Object();
+          var statusMsgObj=new Object();
+          errorMsg=enigmailSvc.parseErrorOutput(errorMsg, statusFlagsObj, statusMsgObj);
+        }
+      }
+    } catch (ex) {}
+    DEBUG_LOG("enigmailMessengerOverlay.js: enigReceiveKeyTerminate: errorMsg="+errorMsg);
+    EnigAlert(EnigGetString("keyImportError")+ errorMsg);
+  }
+  gEnigIpcRequest.close(true);
+
+}
+
+function enigReceiveKeyCancel(progressBar) {
+  DEBUG_LOG("enigmailMessengerOverlay.js: enigReceiveKeyCancel\n");
+
+  var keyRetrProcess = gEnigIpcRequest.pipeTransport;
+
+  if (keyRetrProcess && !keyRetrProcess.isAttached()) {
+    keyRetrProcess.terminate();
+  }
+  gEnigIpcRequest.close(true);
+
+  EnigAlert(EnigGetString("keyImportError")+ EnigGetString("failCancel"));
+
+}
